/*
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.sourceforge.basher;

import java.util.ArrayList;
import java.util.List;

import junit.framework.TestCase;
import org.easymock.MockControl;
import org.easymock.classextension.EasyMock;

import static org.easymock.classextension.EasyMock.*;

/**
 * @author Johan Lindquist
 * @version 1.0
 */
public class BasherTestCase extends TestCase
{
    private List<MockControl> _mockControls = new ArrayList<MockControl>();

    private List<Object> _mocks = new ArrayList<Object>();

    protected void sleep(final long period)
    {
        try
        {
            Thread.sleep(period);
        }
        catch (InterruptedException e)
        {
            fail(e.getMessage());
        }
    }

    public <T> T  createMock(Class<T> clazz)
    {
        final T mock = EasyMock.createMock(clazz);
        _mocks.add(mock);
        return mock;
    }

    public void replayAll()
    {
        for (Object mock : _mocks)
        {
            replay(mock);
        }
    }

    public void verifyAll()
    {
        for (Object mock : _mocks)
        {
            verify(mock);
        }
    }

    public void resetAll()
    {
        for (Object mock : _mocks)
        {
            reset(mock);
        }
    }


    public void verifyAllMockControls()
    {
        for (final MockControl mockControl : _mockControls)
        {
            mockControl.verify();
        }
    }

}
