package net.sourceforge.basher.impl;

import java.util.List;

import junit.framework.TestCase;
import net.sourceforge.basher.*;

/**
 * @author Johan Lindquist
 */
public class TestTaskTree extends TestCase
{
    private NamedTestTask _testTask1;
    private NamedTestTask _testTask2;
    private NamedTestTask _testTask3;
    private NamedTestTask _testTask4;
    private NamedTestTask _testTask5;
    private NamedTestTask _testTask6;
    private NamedTestTask _testTask7;

    private TaskContribution _taskContribution1;
    private TaskContribution _taskContribution2;
    private TaskContribution _taskContribution3;
    private TaskContribution _taskContribution4;
    private TaskContribution _taskContribution5;
    private TaskContribution _taskContribution6;
    private TaskContribution _taskContribution7;

    private TaskConfiguration _taskConfiguration1;
    private TaskConfiguration _taskConfiguration2;
    private TaskConfiguration _taskConfiguration3;
    private TaskConfiguration _taskConfiguration4;
    private TaskConfiguration _taskConfiguration5;
    private TaskConfiguration _taskConfiguration6;
    private TaskConfiguration _taskConfiguration7;

    public void testGetTasksWithWildcard()
    {
        TaskTree taskTree = new TaskTree();
        taskTree.addTask( _taskContribution1 );
        taskTree.addTask( _taskContribution2 );
        taskTree.addTask( _taskContribution3 );
        taskTree.addTask( _taskContribution4 );

        List<TaskContribution> taskList = taskTree.getTasks( "1.2.*" );
        assertNotNull("unexpected null", taskList);
        assertEquals("unexpected size",2,taskList.size());

        taskList = taskTree.getTasks( "1.*" );
        assertNotNull("unexpected null", taskList);
        assertEquals("unexpected size",2,taskList.size());

        taskList = taskTree.getTasks( "*" );
        assertNotNull("unexpected null", taskList);
        assertEquals("unexpected size",4,taskList.size());

    }

    public void testGetTasksWithIncludes()
    {
        TaskTree taskTree = new TaskTree();
        taskTree.addTask( _taskContribution1 );
        taskTree.addTask( _taskContribution2 );
        taskTree.addTask( _taskContribution3 );
        taskTree.addTask( _taskContribution4 );
        taskTree.addTask( _taskContribution5 );
        taskTree.addTask( _taskContribution6 );
        taskTree.addTask( _taskContribution7 );

        taskTree.setIncludes("1.*,0.*");

        List<TaskContribution> taskList = taskTree.getTasks();
        assertNotNull("unexpected null", taskList);
        assertEquals("unexpected size",3,taskList.size());

        assertTrue("expected task not present", taskList.contains(_taskContribution1));
        assertTrue("expected task not present", taskList.contains(_taskContribution2));
        assertTrue("expected task not present", taskList.contains(_taskContribution3));

        taskTree.setIncludes("3.2.3.*,3.2.3.5.*");

        taskList = taskTree.getTasks();
        assertNotNull("unexpected null", taskList);
        assertEquals("unexpected size",2,taskList.size());

        assertTrue("expected task not present", taskList.contains(_taskContribution6));
        assertTrue("expected task not present", taskList.contains(_taskContribution7));

    }

    public void testGetTasksWithIncludesAndExcludes()
    {
        TaskTree taskTree = new TaskTree();
        taskTree.addTask( _taskContribution1 );
        taskTree.addTask( _taskContribution2 );
        taskTree.addTask( _taskContribution3 );
        taskTree.addTask( _taskContribution4 );
        taskTree.addTask( _taskContribution5 );
        taskTree.addTask( _taskContribution6 );
        taskTree.addTask( _taskContribution7 );

        taskTree.setIncludes("1.*,0.1.TestTask*");
        taskTree.setExcludes("1.2.*");

        List<TaskContribution> taskList = taskTree.getTasks();
        assertNotNull("unexpected null", taskList);
        assertEquals("unexpected size",1,taskList.size());

        assertTrue("expected task not present", taskList.contains(_taskContribution3));

    }


    protected void setUp() throws Exception
    {
        super.setUp();
        _testTask1 = new NamedTestTask( "1.2.3.TestTask1" );
        _testTask2 = new NamedTestTask( "1.2.TestTask2" );
        _testTask3 = new NamedTestTask( "0.1.TestTask3" );
        _testTask4 = new NamedTestTask( "TestTask4" );
        _testTask5 = new NamedTestTask( "3.1.2.TestTask5" );
        _testTask6 = new NamedTestTask( "3.2.3.4.TestTask6" );
        _testTask7 = new NamedTestTask( "3.2.3.5.TestTask7" );

        _taskConfiguration1 = new TaskConfiguration();
        _taskConfiguration1.setTaskName(_testTask1.getName());

        _taskConfiguration2 = new TaskConfiguration();
        _taskConfiguration2.setTaskName(_testTask2.getName());

        _taskConfiguration3 = new TaskConfiguration();
        _taskConfiguration3.setTaskName(_testTask3.getName());

        _taskConfiguration4 = new TaskConfiguration();
        _taskConfiguration4.setTaskName(_testTask4.getName());

        _taskConfiguration5 = new TaskConfiguration();
        _taskConfiguration5.setTaskName(_testTask5.getName());

        _taskConfiguration6 = new TaskConfiguration();
        _taskConfiguration6.setTaskName(_testTask6.getName());

        _taskConfiguration7 = new TaskConfiguration();
        _taskConfiguration7.setTaskName(_testTask7.getName());

        _taskContribution1 = new TaskContribution(_testTask1, _taskConfiguration1);
        _taskContribution2 = new TaskContribution(_testTask2, _taskConfiguration2);
        _taskContribution3 = new TaskContribution(_testTask3, _taskConfiguration3);
        _taskContribution4 = new TaskContribution(_testTask4, _taskConfiguration4);
        _taskContribution5 = new TaskContribution(_testTask5, _taskConfiguration5);
        _taskContribution6 = new TaskContribution(_testTask6, _taskConfiguration6);
        _taskContribution7 = new TaskContribution(_testTask7, _taskConfiguration7);

    }
}
