package net.sourceforge.basher.internal.impl;

import java.util.List;
import java.util.ArrayList;
import java.util.Arrays;

import junit.framework.TestCase;
import net.sourceforge.basher.Task;
import net.sourceforge.basher.Phase;
import org.ops4j.gaderian.service.impl.ClassFactoryImpl;
import org.apache.commons.logging.LogFactory;

/**
 * @author Johan Lindquist
 * @version $Revision$
 */
public class TestTaskDecoratorImpl extends TestCase
{
    public TaskDecoratorImpl _taskDecoratorImpl;

    public void testSimpleClassWithSomeTaskMethods()
    {
        final Task task = _taskDecoratorImpl.decorateInstance( new SimpleClassWithSomeTaskMethods() );
        assertNotNull( task );
    }


    public void testDetermineExecuteMethodSingleAnnotation()
    {
        final Task task = _taskDecoratorImpl.decorateInstance( new SingleExecutionMethodAnnotated() );
        assertNotNull( task );
    }

    public void testFullyAnnotatedClass()
    {
        final FullyAnnotatedClass fullyAnnotatedClass = new FullyAnnotatedClass();

        final Task task = _taskDecoratorImpl.decorateInstance( fullyAnnotatedClass );
        assertNotNull( task );

        // Attempt to execute the task
        try
        {
            task.executeTask();
            assertEquals( "bad invoke count", 1, fullyAnnotatedClass.getInvocationCount() );
        }
        catch ( Throwable throwable )
        {
            throwable.printStackTrace();
            fail( throwable.getMessage() );
        }

    }

    public void testSubClassFullyAnnotated()
    {
        final SubClassFullyAnnotatedClass subClassFullyAnnotatedClass = new SubClassFullyAnnotatedClass();
        final Task task = _taskDecoratorImpl.decorateInstance( subClassFullyAnnotatedClass );
        assertNotNull( task );

        // Attempt to execute the task
        try
        {
            task.executeTask();
            assertEquals( "bad invoke count", 1, subClassFullyAnnotatedClass.getInvocationCount() );
        }
        catch ( Throwable throwable )
        {
            throwable.printStackTrace();
            fail( throwable.getMessage() );
        }
    }

    public void testSubSubClassFullyAnnotatedNotPhaseTest()
    {
        final SubSubClassFullyAnnotatedClassNotPhaseTest subSubClassFullyAnnotatedClassNotPhaseTest = new SubSubClassFullyAnnotatedClassNotPhaseTest();
        final Task task = _taskDecoratorImpl.decorateInstance( subSubClassFullyAnnotatedClassNotPhaseTest );
        assertNotNull( task );

        // Attempt to execute the task
        try
        {
            task.executeTask();
            assertEquals( "bad invoke count", 1, subSubClassFullyAnnotatedClassNotPhaseTest.getInvocationCount() );
        }
        catch ( Throwable throwable )
        {
            throwable.printStackTrace();
            fail( throwable.getMessage() );
        }


    }

    public void testDetermineExecuteMethodMoreThanOneAnnotation()
    {
        try
        {
            _taskDecoratorImpl.decorateInstance( new MoreThanOneExecutionMethodAnnotated() );
            fail( "unreachable" );
        }
        catch ( Exception e )
        {
            // Should happen
            assertEquals( "bad message", "Found more than 1 execute method", e.getMessage() );
        }

    }

    public void testDetermineExecuteMethodAnnotatedMethodNotEmptyParams()
    {
        try
        {
            _taskDecoratorImpl.decorateInstance( new ExecutionMethodPresentNotEmptyParamsAnnotated() );
            fail( "unreachable" );
        }
        catch ( Exception e )
        {
            // Should happen
            assertEquals( "bad message", "Execution method 'doit'  must not take any parameters", e.getMessage() );
        }
    }

    public void testDetermineExecuteMethodNoAnnotationDefaultMethodPresent()
    {
        final Task task = _taskDecoratorImpl.decorateInstance( new ExecutionMethodPresent() );
        assertNotNull( task );
    }

    public void testDetermineExecuteMethodNoAnnotationDefaultMethodNotPresent()
    {
        try
        {
            _taskDecoratorImpl.decorateInstance( new NoExecutionMethodPresent() );
            fail( "unreachable" );
        }
        catch ( Exception e )
        {
            // Should happen
            assertEquals( "bad message", "Could not find executeTask", e.getMessage() );
        }
    }

    public void testDetermineExecuteMethodNoAnnotationDefaultMethodNotEmptyParams()
    {
        try
        {
            _taskDecoratorImpl.decorateInstance( new ExecutionMethodPresentNotEmptyParams() );
            fail( "unreachable" );
        }
        catch ( Exception e )
        {
            // Should happen
            assertEquals( "bad message", "Could not find executeTask", e.getMessage() );
        }
    }

    protected void setUp() throws Exception
    {
        super.setUp();
        _taskDecoratorImpl = new TaskDecoratorImpl();
        final ClassFactoryImpl classFactory = new ClassFactoryImpl();
        _taskDecoratorImpl.setClassFactory( classFactory );
        _taskDecoratorImpl.setLog( LogFactory.getLog( TaskDecoratorImpl.class ) );
    }
}
